using MediatR;
using Sledgemapper.Api.Commands;
using Sledgemapper.Api.Infrastructure.Data;
using Sledgemapper.Shared.Entities;
using Sledgemapper.Api.Notifications;
using System.Linq;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;

namespace Sledgemapper.Api.Handlers
{
    public class  DeleteOverlayCommandHandler : IRequestHandler<DeleteOverlayCommand, bool>
    {
        private readonly SledgemapperDbContext _dbcontext;

        private readonly IMediator _mediator;

        public DeleteOverlayCommandHandler(IMediator mediator, SledgemapperDbContext dbcontext) { _dbcontext = dbcontext; _mediator = mediator; }

        public async Task<bool> Handle(DeleteOverlayCommand notification, CancellationToken cancellationToken)
        {
            var jsonString = JsonSerializer.Serialize(notification.Overlay);
            var session = _dbcontext.Sessions.First(m=>m.SessionName== notification.SessionName);

            _dbcontext.MapLogs.Add(new Models.MapLog
            {
                Operation = "D",
                SessionId = session.SessionId,
                Type = "O",
                Timestamp = notification.Timestamp,
                Object = jsonString,
                UserId=notification.UserId
            });

            await _dbcontext.SaveChangesAsync();
            await _mediator.Publish(new DeleteOverlayNotification(session, notification.Overlay, notification.UserId));
            return true;
        }
    }
}
