using MediatR;
using Sledgemapper.Api.Commands;
using Sledgemapper.Api.Notifications;
using Sledgemapper.Api.Infrastructure.Data;
using Sledgemapper.Shared.Entities;
using System.Linq;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;

namespace Sledgemapper.Api.Handlers
{
    public class NewOverlayCommandHandler : IRequestHandler<NewOverlayCommand, bool>
    {
        private readonly SledgemapperDbContext _dbcontext;

        private readonly IMediator _mediator;

        public NewOverlayCommandHandler(IMediator mediator, SledgemapperDbContext dbcontext) { _dbcontext = dbcontext; _mediator = mediator; }

        public async Task<bool> Handle(NewOverlayCommand notification, CancellationToken cancellationToken)
        {
            var jsonString = JsonSerializer.Serialize(notification.Overlay);
            var session = _dbcontext.Sessions.First(m => m.SessionName == notification.SessionName);
            _dbcontext.MapLogs.Add(new Models.MapLog
            {
                Operation = "N",
                SessionId = session.SessionId,
                Type = "O",
                Timestamp = notification.Timestamp,
                Object = jsonString,
                UserId = notification.UserId,
            });
            await _dbcontext.SaveChangesAsync();
            await _mediator.Publish(new NewOverlayNotification(session, notification.Overlay, notification.UserId));
            return true;
        }
    }
}
